/* errors.c -- Dune Webserver
 * Copyright (C) 1998 Baris Mestanogullari <mestanba@martin.luther.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include "errors.h"
#include "config.h"

void sendBadRequest(int head) {
  sprintf(buffer, "HTTP/1.0 400 Bad Request\n");
  sprintf(buffer2, "Date: %s\n", (char *)currentTime());
  strcat(buffer, buffer2);
  sprintf(buffer2, "Server: Dune/%s\n", VERSION);
  strcat(buffer, buffer2);
  strcat(buffer, "Connection: close\n");
  strcat(buffer, "Content-Type: text/html\n\n");
  write(cfd, buffer, strlen(buffer));
  if(head) return;
#ifdef BADREQUESTURL
  sendFile(BADREQUESTURL, 0);
  return;
#endif
  sprintf(buffer, "<HTML><HEAD>\n<TITLE>400 Bad Request</TITLE>\n\
</HEAD><BODY>\n<H1>Bad Request</H1>\nYour browser sent a request \
that this server could not understand.<P>\n</BODY></HTML>\n");
  write(cfd, buffer, strlen(buffer));
}

void sendForbidden(char *url, int head) {
  sprintf(buffer, "HTTP/1.0 403 Forbidden\n");
  sprintf(buffer2, "Date: %s\n", (char *)currentTime());
  strcat(buffer, buffer2);
  sprintf(buffer2, "Server: Dune/%s\n", VERSION);
  strcat(buffer, buffer2);
  strcat(buffer, "Connection: close\n");
  strcat(buffer, "Content-Type: text/html\n\n");
  write(cfd, buffer, strlen(buffer));
  if(head) return;
#ifdef FORBIDDENURL
  sendFile(FORBIDDENURL, 0);
  return;
#endif
  sprintf(buffer, "<HTML><HEAD>\n<TITLE>403 Forbidden</TITLE>\n\
</HEAD><BODY>\n<H1>Forbidden</H1>\nYou don't have permissions to \
access %s on this server.<P>\n</BODY></HTML>\n", url);
  write(cfd, buffer, strlen(buffer));
}

void sendNotFound(char *url, int head) {
  sprintf(buffer, "HTTP/1.0 404 File Not Found\n");
  sprintf(buffer2, "Date: %s\n", (char *)currentTime());
  strcat(buffer, buffer2);
  sprintf(buffer2, "Server: Dune/%s\n", VERSION);
  strcat(buffer, buffer2);
  strcat(buffer, "Connection: close\n");
  strcat(buffer, "Content-Type: text/html\n\n");
  write(cfd, buffer, strlen(buffer));
  if(head) return;
#ifdef NOTFOUNDURL
  sendFile(NOTFOUNDURL, 0);
  return;
#endif
  sprintf(buffer, "<HTML><HEAD>\n<TITLE>404 File Not Found</TITLE>\n\
</HEAD><BODY>\n<H1>File Not Found</H1>\nThe requested URL %s was not \
found on this server.<P>\n</BODY></HTML>\n", url);
  write(cfd, buffer, strlen(buffer));
}

void sendMovedPermanently(char *url, int head) {
  sprintf(buffer, "HTTP/1.1 301 Moved Permanently\n");
  sprintf(buffer2, "Date: %s\n", (char *)currentTime());
  strcat(buffer, buffer2);
  sprintf(buffer2, "Server: Dune/%s\n", VERSION);
  strcat(buffer, buffer2);
  sprintf(buffer2, "Location: %s/\n", url);
  strcat(buffer, buffer2);
  strcat(buffer, "Connection: close\n");
  strcat(buffer, "Content-Type: text/html\n\n");
  write(cfd, buffer, strlen(buffer));
  if(head) return;

  sprintf(buffer, "<HTML><HEAD>\n<TITLE>301 Moved Permanently\
</TITLE>\n</HEAD><BODY>\n<H1>Moved Permanently</H1>\nThe document \
has moved <A HREF=\"%s/\">here</A>.<P>\n</BODY></HTML>\n", url);
  write(cfd, buffer, strlen(buffer));
}

void sendNotImplemented() {
  sprintf(buffer, "HTTP/1.0 501 Method Not Implemented\n");
  sprintf(buffer2, "Date: %s\n", (char *)currentTime());
  strcat(buffer, buffer2);
  sprintf(buffer2, "Server: Dune/%s\n", VERSION);
  strcat(buffer, buffer2);
  strcat(buffer, "Allow: GET, HEAD\n");
  strcat(buffer, "Connection: close\n");
  strcat(buffer, "Content-Type: text/html\n\n");
  write(cfd, buffer, strlen(buffer));

  sprintf(buffer, "<HTML><HEAD>\n<TITLE>501 Method Not Implemented\
</TITLE>\n</HEAD><BODY>\n<H1>Method Not Implemented</H1>\nThis \
server only supports GET and HEAD.<P>\n</BODY></HTML>\n");
  write(cfd, buffer, strlen(buffer));
}
