/* index.c -- Dune Webserver
 * Copyright (C) 1998 Baris Mestanogullari <mestanba@martin.luther.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <grp.h>
#include <pwd.h>

#include "index.h"
#include "config.h"

char *permissions(unsigned short mode, char *mod)
{
  /* Assume octal 0000 for permissions by default */
  strcpy(mod, "----------\0");

  /* Get what type of file it is */
  if(S_ISDIR(mode)) mod[0]='d';
  if(S_ISLNK(mode)) mod[0]='l';
  if(S_ISCHR(mode)) mod[0]='c';
  if(S_ISBLK(mode)) mod[0]='b';
  if(S_ISFIFO(mode)) mod[0]='p';
  if(S_ISSOCK(mode)) mod[0]='s';

  /* User permissions */
  if(S_IRUSR&mode) mod[1]='r';
  if(S_IWUSR&mode) mod[2]='w';
  if(S_IXUSR&mode) mod[3]='x';
  if(S_ISUID&mode) mod[3]='s'; /* Overwrite 'x' with an 's' if SUID */

  /* Group permissions */
  if(S_IRGRP&mode) mod[4]='r';
  if(S_IWGRP&mode) mod[5]='w';
  if(S_IXGRP&mode) mod[6]='x';
  if(S_ISGID&mode) mod[6]='s'; /* Overwrite 'x' with an 's' if SGID */

  /* Other people's permissions */
  if(S_IROTH&mode) mod[7]='r';
  if(S_IWOTH&mode) mod[8]='w';
  if(S_IXOTH&mode) mod[9]='x';

  return mod;
}

void sendIndex(char *directory, char *pageRequested, int head)
{
  struct direct *entry;
  struct stat fentry;
  struct passwd *pwent;
  struct group *grent;
  char mod[11], size[11], buf[32];
  int count=0;
  DIR *dp;

  if(strcmp(pageRequested, "/")) pageRequested[strlen(pageRequested)-1]='\0';
  dp=opendir(directory);

  sprintf(buffer, "HTTP/1.0 200 OK\r\nDate: %s\nServer: Dune/%s\r\n\
Connection: close\r\n", currentTime(), VERSION);
  write(cfd, buffer, strlen(buffer));
  sprintf(buffer, "Content-Type: text/html\r\n\r\n");
  write(cfd, buffer, strlen(buffer));
  if(head) return;

  sprintf(buffer, "<HTML><HEAD>\r\n<TITLE>Index of %s</TITLE>\r\n</HEAD><BODY>\
\r\n<H1>Index of %s</H1>\r\n<BR>\r\n<PRE>\r\nPermissions  Owner     Group     Size\
          Last Modified   Name\r\n</PRE>\r\n<HR>\r\n<BR><B>\r\n<PRE>\r\n", 
	  pageRequested, pageRequested);
  write(cfd, buffer, strlen(buffer));

  chdir(directory);
  for(entry=readdir(dp); entry!=NULL; entry=readdir(dp)) {
    if(((SHOWHIDDEN || entry->d_name[0]!='.') && strcmp(entry->d_name, "."))
       || !strcmp(entry->d_name, "..")) {
      stat(entry->d_name, &fentry);
      sprintf(buffer, "%s   ", permissions(fentry.st_mode, mod));
      write(cfd, buffer, strlen(buffer));
      if((pwent=getpwuid(fentry.st_uid))==NULL) {
	sprintf(buffer, "%d", fentry.st_uid);
	write(cfd, buffer, strlen(buffer));
	for(count=0;count<(10-strlen((char *)(int)fentry.st_uid));count++)
	  write(cfd, " ", 1);
      }
      else {
	sprintf(buffer, "%s", pwent->pw_name);
	write(cfd, buffer, strlen(buffer));
	for(count=0;count<(10-strlen(pwent->pw_name));count++)
	  write(cfd, " ", 1);
      }
      if((grent=getgrgid(fentry.st_gid))==NULL) {
	sprintf(buffer, "%d",fentry.st_gid);
	write(cfd, buffer, strlen(buffer));
	for(count=0;count<(10-strlen((char *)(int)fentry.st_gid));count++)
	  write(cfd, " ", 1);
      }
      else {
	sprintf(buffer, "%s",grent->gr_name);
	write(cfd, buffer, strlen(buffer));
	for(count=0;count<(10-strlen(grent->gr_name));count++)
	  write(cfd, " ", 1);
      }
      sprintf(buf,"%d",(int)fentry.st_size);
      for(count=0;count<10;count++)
	size[count]=' ';
      
      /* I want to have my filesize right-aligned */
      for(count=0;count<strlen(buf);count++)
	size[11-strlen(buf)+count]=buf[count];

      size[11]='\0';
      sprintf(buffer, "%s   ",size);
      write(cfd, buffer, strlen(buffer));
      strncpy(buf, (char *)ctime(&fentry.st_mtime),16);
      strcpy(buf, buf+4);
      buf[12]='\0';
      sprintf(buffer, "%s    ",buf);
      write(cfd, buffer, strlen(buffer));

      if(!strcmp(entry->d_name, "..")) {
        if(!strcmp(pageRequested, "/"))
          sprintf(buffer, "<A HREF=\"/\">Parent directory</A>\r\n");
        else
	  sprintf(buffer, "<A HREF=\"..\">Parent directory</A>\r\n");
	write(cfd, buffer, strlen(buffer));
      }
      else {
        char buf[256];
	sprintf(buffer, "<A HREF=\"%s", entry->d_name);
        if(mod[0]=='d') strcat(buffer, "/");
	sprintf(buf, "\">%s", entry->d_name);
        strcat(buffer, buf);
        if(mod[0]=='d') strcat(buffer, "/");
        strcat(buffer, "</A>\r\n");
	write(cfd, buffer, strlen(buffer));
      }
    }
  }
  write(cfd, "</PRE>\r\n</BODY></HTML>\r\n", 25);
  closedir(dp);
}
